package data;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.StringTokenizer;



/**
 * Deze klasse geeft een bericht terug.
 * Deze kan opgebouwd worden en/of opgevraagd worden. 
 * 
 * @version 0.30
 * TODO Settings kunnen instellen (paden etc.)
 * TODO Kunnen verwijderen/verplaatsen mail na ophalen van POP3.
 * TODO Globale functie splitsen domein en user. Nu te vaak repeterende code.
 */
public class Message {
	
	private String sender;
	private String receiver;
	private String data;
	private String date;
	private String recDomain;
	
	private static String dataPath = "data/";
	
	/**
	 * Constructor
	 */
	public Message()
	{
		setDate();
	}
	
	/**
	 * Stelt de afzender van het bericht in.
	 * @param address
	 */
	public void setFrom(String address){
		sender = address;		
	}
	
	/**
	 * Geeft de afzender terug.
	 * @return afzener
	 */
	public String getFrom(){	
		return sender;
	}	
	
	/**
	 * Stelt de geadresseerde in.
	 * @param address
	 */
	public void setTo(String address){
		setDomain(address);
		receiver = address;
	}
	
	/**
	 * Stelt het domain in, voor de ontvanger
	 * @param address
	 */
	public void setDomain(String addressTo){
		recDomain = addressTo.substring(addressTo.indexOf("@")+1);
		recDomain = recDomain.replace(">", "");
	}

	/**
	 * Geeft het ontvanger domein terug.
	 * @return domein
	 */
	public String getDomain(){
		return recDomain;
	}
	
	/**
	 * Geeft de geadresseerde terug
	 * @return
	 */
	public String getTo(){
		return receiver;
	}
	
	/**
	 * Stelt de data van het bericht in.
	 * @param data
	 */
	public void setData(String data){
		this.data = data; 		
	}
	
	/**
	 * Geeft de inhoud van het mailtje retour.
	 * @return
	 */
	public String getData(){
		return data;
	}

	/**
	 * Stelt de datum en tijd in.
	 *
	 */ 
	private void setDate(){
		
		String DATE_FORMAT_NOW = "yyyy-MM-dd HH:mm:ss";
		Calendar cal = Calendar.getInstance();
	    SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
	    date = sdf.format(cal.getTime());
		
	}
	
	/**
	 * Geeft de datum als string terug.
	 * @return
	 */
	public String getDate(){
		return date;
	}
	
	/**
	 * Schrijft het bericht weg en geeft aan of dit gelukt is.
	 * @param location
	 * @return
	 */
	public boolean writeMessage() {
		String path;
		boolean msgExists;
		String filename;
		String file = "";
		int i;
		File chkFile;
		//zorgen dat emailadres word gesplitst in user en domain
		StringTokenizer st = new StringTokenizer(receiver, "@");
		String user = st.nextToken();
		String domain = st.nextToken();
				
		//Sommige cliënts sturen de mail naar zo'n adres "<test@wereld.com>".
		user = user.replace("<", "");
		domain = domain.replace(">", "");
				
		user = user.trim();
		domain = domain.trim();
		
		//persoonlijke map: mailserver\data\mail\domein\gebruiker\*.txt
		//Misschien controleren op userexist?
		if (User.domainExists(receiver)){
			path = dataPath + "/mail/" + domain.toLowerCase() + "/" + user.toLowerCase();
		}
		else{
			path = dataPath + "/relay/";
		}
		
		try {
			File dir = new File(path);
			//als de map niet bestaat
			if (!dir.exists()) {
	            dir.mkdirs();
	        }
			
			//Unieke bestandsnaam voor het bericht verzinnen
			msgExists = true;
			filename = "message_";
			i = 0; 
			while (msgExists){
				i++;
				file = path + "/" + filename + i + ".txt";
				chkFile = new File(file);
				if (!chkFile.exists()){
					msgExists = false;
				}				
			}
			//opslaan
			PrintStream out = new PrintStream(new FileOutputStream(file));
			out.println("" + sender);
			out.println("" + receiver);
			out.println("" + date);
			out.println("" + data);
			out.flush();
			out.close();
			return true;
		} catch (Exception e) {
			System.out.println("" + e.getMessage());
			//als het wegschrijven niet gelukt is
			return false;
		}
		
	}
	
	/**
	 * Probeert een bericht te verwijderen.
	 * Doet dit a.d.h.v. email box met bericht nummer.
	 * 
	 * @param email
	 * @param msgID
	 * @return true als het gelukt is
	 */
	public static void delMessage(String email, ArrayList msgID){
		String path;
		File dir;
		File[] files;
		
		//zorgen dat emailadres word gesplitst in user en domain
		StringTokenizer st = new StringTokenizer(email, "@");
		String user = st.nextToken();
		String domain = st.nextToken();
		
		//Juiste path selecteren
		path = dataPath + "/mail/" + domain.toLowerCase() + "/" + user.toLowerCase();
		dir = new File(path);
		files = dir.listFiles();
		
		Iterator it = msgID.iterator();
		while (it.hasNext()){
			// -1 Gebruikers beginnen bij 1, de array bij 0.
			files[(Integer) it.next() - 1].delete();			
		}		
	}
	
	/**
	 * Probeert een bericht in te laden en geeft aan of dit gelukt is.
	 * Haalt het bericht op a.d.h.v. emailbox met bericht nummer.
	 * 
	 * @param email, msgID
	 * @return Message.
	 */
	public static Message getMessage(String email, int msgID){
		Message msg = null;
		String path;
		File dir;
		File[] files;
		
		msgID = msgID - 1; //Gebruikers beginnen bij 1, array's bij 0.
		
		//zorgen dat emailadres word gesplitst in user en domain
		StringTokenizer st = new StringTokenizer(email, "@");
		String user = st.nextToken();
		String domain = st.nextToken();
		
		//Pad waar de berichten opgehaald dienen te worden.
		path = dataPath + "/mail/" + domain.toLowerCase() + "/" + user.toLowerCase();
		dir = new File(path);
		files = dir.listFiles();
		
		//Uiteindelijk bericht wordt via de andere functie opgehaald!
		msg = Message.getMessage(files[msgID]);
		
		return msg;
	}
	
	/**
	 * Haalt een bericht op a.d.h.v.d. bestandsnaam.
	 * @param bestand
	 * @return Message
	 */
	public static Message getMessage(File bestand){
		Message msg = new Message();
		String tmp, tmpData;
		BufferedReader input;
		int i;
		try{
			input = new BufferedReader(new FileReader(bestand));
			i = 0;
			tmpData = "";
			while ((tmp = input.readLine()) != null ){
				//LET OP! Afhankelijk van de afgesproken volgorde!
				if (i == 0){
					msg.setFrom(tmp);
				}
				else if (i == 1){
					msg.setTo(tmp);
				}
				//Op 2 zit de datum hier doen we nu niks mee.
				else if (i > 2){
					tmpData = tmpData + tmp + System.getProperty("line.separator");
				}
				i++;
			}
			msg.setData(tmpData);
			input.close();
			return msg;
		}
		catch(Exception ex){
			ex.printStackTrace();
			return null;
		}		
	}	
	
	/**
	 * Geeft een berichtenlijst terug.
	 * Geef 0 mee voor de hele lijst.
	 * @param email, 0 geeft hele lijst, anders een specifiek bericht
	 * @return null bij geen berichten, anders lijst gescheiden door een line.separator.
	 */
	public static String getMessageList(String email, int msgID){
		String path;
		File dir;
		File[] files;
		int i, tmp;
		String result = "";
		String D = "";
		
		//zorgen dat emailadres word gesplitst in user en domain
		StringTokenizer st = new StringTokenizer(email, "@");
		String user = st.nextToken();
		String domain = st.nextToken();
		
		
		
		//Pad waar de berichten opgehaald dienen te worden.
		path = dataPath + "/mail/" + domain.toLowerCase() + "/" + user.toLowerCase();
		dir = new File(path);
		if (!dir.exists()){
			dir.mkdirs();
		}
		files = dir.listFiles();
		
		if ((files != null) && (msgID == 0)){
			//Loop door de files heen.
			for (i=0; i < files.length; i++){
				tmp = i+1; //Gebruikers beginnen bij 1 met tellen.
				result = result + tmp + " " + files[i].length() + "\r" + System.getProperty("line.separator");
			}
			result = result  + D + ".\r";
		}
		else if (msgID > 0){
			//result = result + (msgID-1) + " " + files[msgID-1].length() + "\r" + System.getProperty("line.separator");
			result = files[msgID-1].length() + " octets";// + System.getProperty("line.separator");
		}
		else{
			result = ".\r";
		}
		return result;
	}
	
	/**
	 * Retourneert de statistieken.
	 * @param email
	 * @return
	 */
	public static String getMessageStat(String email){
		int msgCount = 0;
		int byteCount = 0;
		String tmp;
		String parser = " ";
		
		StringTokenizer st = new StringTokenizer(Message.getMessageList(email, 0), System.getProperty("line.separator"));
		while (st.hasMoreTokens()){
			//Token geeft "1 408"
			tmp = st.nextToken();
			tmp = tmp.substring(tmp.indexOf(parser)+1, tmp.length());
			tmp = tmp.trim();
			if (!tmp.substring(0,1).equals(".")){
				byteCount = byteCount + Integer.parseInt(tmp);
				msgCount = msgCount + 1;
			}
		}
		//return msgCount + " berichten (" + byteCount + " octets)";
		return msgCount + " " + byteCount;
	}
	
	/**
	 * Geeft een lijst met bericht ID's incl. de Unieke ID's
	 * @return
	 */
	public static String getMessageUID(String email, int msgID){
		String path;
		File dir;
		File[] files;
		int i, tmp;
		String result = "";
		
		//zorgen dat emailadres word gesplitst in user en domain
		StringTokenizer st = new StringTokenizer(email, "@");
		String user = st.nextToken();
		String domain = st.nextToken();
		
		//Pad waar de berichten opgehaald dienen te worden.
		path = dataPath + "/mail/" + domain.toLowerCase() + "/" + user.toLowerCase();
		dir = new File(path);
		files = dir.listFiles();
		
		if (msgID == 0){
			//Loop door de files heen.
			for (i=0; i < files.length; i++){
				tmp = i+1; //Gebruikers beginnen bij 1 met tellen.
				result = result + tmp + " " + generateUID(files[i].length(), files[i].lastModified()) + System.getProperty("line.separator");
				//result = result + tmp + " " + "0000000" + (i+1) + "\r" + System.getProperty("line.separator");
			}
		}
		else{ 	//MSGID - 1 want Array begint bij 0!
			result = msgID + " " + generateUID(files[msgID-1].length(), files[msgID-1].lastModified()) ;//+ System.getProperty("line.separator");
		}

		//result = result + ".";		
		
		return result;
	}
	
	/**
	 * Genereert een UID voor een bericht.
	 * UID's worden door mailcliënts gebruikt, om te controleren of een bericht al is binnengehaald of niet.
	 * UID's zijn gegenereerd a.d.h.v. lastmodified en bestandsgrootte, dat is voorlopig uniek genoeg. :-)
	 * @param Size
	 * @param name
	 * @return
	 */
	private static String generateUID(long size, long date){
		Long tmpUID;
		int intUID;
		String uid = "";
		
		tmpUID = size * date;
		
		//Long is veel te groot en omzetten naar int
		tmpUID = tmpUID / 1000;
		intUID = new Long(tmpUID).intValue();
		
		//int naar String
		uid = Integer.toString(intUID);
		
		if (uid.length() > 8){
			uid = uid.substring(uid.length() - 8);
		}
		else{
			//String evt. aanvullen met 0 tot er een byte aan getallen staat. "00000000"
			while (uid.length() < 8){
				uid = "0" + uid; 
			}
		}
		
		return uid;
	}
	
}
