package pop3Server;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;
import java.util.logging.Logger;


/**
 * Deze klasse luistert constant of er nieuwe verbindingen gemaakt worden.
 */
public class POP3Server extends Thread{

	//Declaratie variabelen
	private boolean acceptClients = true;
	private int clientID = 1;
	private int port;
	private ServerSocket server;
	
	private String hostname;
	private static HashSet activeClients = new HashSet();
	
	private Logger log;
	
	/**
	 * Constructor voor de POP3Server
	 * 
	 * @param hostname
	 * @param port
	 */
	public POP3Server(String hostname, int port){
		this.hostname = hostname;
		this.port = port;
		log = Logger.getLogger("POP3Server");
		log.info("POP3Server gestart.");				
	}
	
	/**
	 * Deze methode start de Pop3Server in een aparte thread.
	 */
	public void run(){
		try{
			server = new ServerSocket(port);
			log.info("Luisteren naar clients op poort: "+port);
			//Blijven accepteren van clints
			while(acceptClients){
				Socket client = server.accept();
				POP3ClientHandler newClient = new POP3ClientHandler(client, hostname, clientID++);
				activeClients.add(newClient);
				log.info("Nieuwe client (ID:" + (clientID-1) + ") verbonden met poort " + client.getLocalPort() + " met ip: " + client.getInetAddress() + " externe poort: " + client.getPort());
				newClient.start();
				//Pauze om andere threads de kans te geven?
				Thread.sleep(1500);
			}
		}	
		catch (Exception ex){
			log.warning("Fout bij openen poort.");
			ex.printStackTrace();
		}
	}
	
	/**
	 * Wel of niet accepteren van nieuwe cliënts.
	 * @param active
	 */
	public void setAcceptClients(boolean acceptClients){
		this.acceptClients = acceptClients;
	}
	
	/**
	 * Geeft als waarde terug of er wel of geen cliënts geaccepteerd worden.
	 * @return acceptClients
	 */
	public boolean getAcceptClients(){
		return acceptClients;
	}
	
	/**
	 * Retourneert het aantal actieve cliënts
	 * @return
	 */
	public Set getActiveClients(){
		return activeClients;
	}
	
}
