package smtpServer;

import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;
import java.util.logging.*;

/**
 * Deze klasse verzorgt alle verbindingen met de cliënts.
 * Er wordt constant geluisterd of er een connectie gemaakt wordt.
 * 
 * Als er een connectie gemaakt wordt, dan wordt deze in een nieuwe thread afgehandeld
 * m.b.v. de SMTPClientHandler.
 * 
 * @version 0.50
 * TODO Logging implementeren.
 * TODO Instellingen kunnen doorgeven.
 */
public class SMTPServer extends Thread{

	//Declaratie variabelen
	private boolean acceptClients = true;
	private int clientID = 1;
	private int port;
	private ServerSocket server;
	
	private String hostname;
	private static HashSet activeClients = new HashSet();
	
	private Logger log;
	
	/**
	 * Constructor voor de SMTPServer
	 * 
	 * @param hostname
	 * @param port
	 */
	public SMTPServer(String hostname, int port){
		this.hostname = hostname;
		this.port = port;
		log = Logger.getLogger("SMTPServer");
		log.info("SMTPServer gestart.");		
	}
	
	/**
	 * Deze methode start de SMTPServer in een aparte thread.
	 * En luisterd continue of er verbindingen gemaakt worden.
	 */
	public void run(){
		try{
			log.info("Luisteren naar clients op poort: "+port);
			server = new ServerSocket(port);
			//Blijven accepteren van cliënactivets
			while(acceptClients){
				Socket client = server.accept();
				SMTPClientHandler newClient = new SMTPClientHandler(client, hostname, clientID++);
				activeClients.add(newClient);
				log.info("Nieuwe client (ID:" + (clientID-1) + ") verbonden met poort " + client.getLocalPort() + " met ip: " + client.getInetAddress() + " externe poort: " + client.getPort());
				newClient.start();
			}
		}	
		catch (Exception ex){
			log.warning("Fout bij openen poort.");
			ex.printStackTrace();
		}
	}
	
	/**
	 * Wel of niet accepteren van nieuwe cliënts.
	 * @param active
	 */
	public void setAcceptClients(boolean acceptClients){
		this.acceptClients = acceptClients;
	}
	
	/**
	 * Geeft als waarde terug of er wel of geen cliënts geaccepteerd worden.
	 * @return acceptClients
	 */
	public boolean getAcceptClients(){
		return acceptClients;
	}
	
	/**
	 * Retourneert het aantal actieve cliënts
	 * @return activeClients
	 */
	public Set getActiveClients(){
		return activeClients;
	}
	
}
